package File::Tags;

our $VERSION = '0.01';

use strict;
use warnings;
use DBI;
use Cwd;

sub new {
	my $class = shift;
	my $self = {};
	bless($self,$class);

	# TODO: Read config... for now, just use $HOME/.ptags/tags as database
	# NOTE: Can be insecure, if people set $HOME to random values(?)
	return $self unless @_;
	my $db = shift; #$ENV{HOME}.'/.ptags/tags';
	# Set dbh
	$self->dbh("dbi:SQLite:$db");
	return $self;
}

sub dbh {
	my $self = shift;
	$self->{DBH} = DBI->connect(shift) if @_;
	return $self->{DBH};
}

sub build_table {
	my $self = shift;
	# Currently VERY inneficient...
	# File names stored with abs_path()
	my $sth = $self->dbh->prepare(<<SQL);
CREATE TABLE IF NOT EXISTS tags (
	id	INTEGER PRIMARY KEY AUTOINCREMENT,
	filename	TEXT,
	tag	TEXT,
	UNIQUE (filename,tag)
);
SQL

	$sth->execute or warn $self->dbh->errstr;
	$sth = $self->dbh->prepare(<<SQL);
CREATE TABLE IF NOT EXISTS colors (
	id	INTEGER PRIMARY KEY AUTOINCREMENT,
	tag TEXT,
	color TEXT,
	UNIQUE (tag)
);
SQL
	$sth->execute or warn $self->dbh->errstr;
}

sub add_tag {
	# Tags file $_[1] with tag $_[2 .. $#_]
	my ( $self, $filename, @tags ) = @_;
	$filename = Cwd::abs_path($filename);
	my $sth = $self->dbh->prepare(
		'INSERT INTO tags (filename,tag) VALUES(?,?)');
	foreach my $tag (@tags) {
		$sth->execute($filename,$tag) or warn $self->dbh->errstr;
	}
	return 1;
}

sub tags_full_path {
	# Get all tags for file $_[1]
	# Returns as array ref of tags
	my ( $self, @filenames ) = @_;
	my %out;
	my $sth = $self->dbh->prepare(
		'SELECT tag FROM tags WHERE filename=?');
	foreach my $filename (@filenames) {
		$filename = Cwd::abs_path($filename);
		$sth->execute($filename) or warn $self->dbh->errstr && return;
		while ( my @row = $sth->fetchrow_array ) {
			push (@{ $out{$filename} }, shift(@row));
		}
	}
	return \%out;
}

sub get_tags {
	# Get tags for a single file
	my ( $self, $filename ) = @_;
	my @out;
	my $sth = $self->dbh->prepare('SELECT tag FROM tags WHERE filename=?');
	$filename = Cwd::abs_path($filename);
	$sth->execute($filename) or warn $self->dbh->errstr && return;
	while ( my @row = $sth->fetchrow_array ) {
		push (@out, shift(@row) );
	}
	return \@out;
}

sub all_with_tag {
	# Takes one or more tags as argument
	# EXACT MATCHES ONLY
	# TODO: Write method that can take regexp as argument...
	# returns array ref
	my ( $self, @tags ) = @_;
	my @out;
	my $st = 'SELECT filename FROM tags WHERE tag=?';
	for ( my $c = 0; $c < $#tags; $c++ ) {
		$st .= ' INTERSECT SELECT filename FROM tags WHERE tag=?';
	}
	my $sth = $self->dbh->prepare($st);
	$sth->execute(@tags) or warn $self->dbh->errstr && return;
	while ( my @row = $sth->fetchrow_array ) {
		push (@out, shift(@row) );
	}
	return \@out;
}

sub full_report {
	# Returns hashref like $filename => \@tags
	my $self = shift;
	my $sth = $self->dbh->prepare('SELECT filename,tag FROM tags');
	$sth->execute or warn $self->dbh->errstr && return;
	my %out;
	while (my @row = $sth->fetchrow_array) {
		my $filename = shift(@row);
		my $tag = shift(@row);
		push(@{ $out{$filename} },$tag);
	}
	return \%out;
}

sub all_files {
	# Returns array ref of all files...
	my $self = shift;
	my $sth = $self->dbh->prepare('SELECT DISTINCT filename FROM tags');
	$sth->execute or warn $self->dbh->errstr && return;
	my @out;
	while (my @row = $sth->fetchrow_array) {
		push(@out,shift(@row));
	}
	return \@out;
}

sub all_tags {
	# Returns array ref of all tags...
	my $self = shift;
	return $self->{ALL_TAGS} if defined $self->{ALL_TAGS};
	my $sth = $self->dbh->prepare('SELECT DISTINCT tag FROM tags');
	$sth->execute or warn $self->dbh->errstr && return;
	while (my @row = $sth->fetchrow_array) {
		push(@{ $self->{ALL_TAGS} },shift(@row));
	}
	return $self->{ALL_TAGS};
}

sub delete_file {
	# Deletes all tags for a given file
	my $self = shift;
	my $file = shift;
	my $sth = $self->dbh->prepare('DELETE FROM tags WHERE filename=?');
	$sth->execute($file) or warn $self->dbh->errstr && return;
	return 1;
}

sub delete_tag {
	# Deletes specified tag from optional specified files.
	# Without specified files, deletes tag from all files.
	my $self = shift;
	my $tag = shift;
	my @files;
	@files = @_ if @_;
	@files = map ( Cwd::abs_path($_), @files );
	my $st = 'DELETE FROM tags WHERE tag=?';
	if ( @files ) {
		$st .= ' AND ( filename=?';
		for ( my $c = 0; $c < $#files; $c++ ) {
			$st .= ' OR filename=?';
		}
		$st .= ' ) ';
	}
	my $sth = $self->dbh->prepare($st);
	$sth->execute($tag,@files) or warn $self->dbh->errstr && return;
	return 1;
}

sub color_enabled {
	my $self = shift;
	$self->{COLOR} = 1 if @_;
	return $self->{COLOR};
}

sub colored_tag {
	# Takes tag as input, returns Term::ANSIColor->colored($tag,$self->color);
	my $self = shift;
	my $tag = shift;
	require Term::ANSIColor;
	return Term::ANSIColor::colored( $tag, $self->color($tag) );
}

sub colored_tags {
	# Same as colored_tag, but takes array ref as input
	my $self = shift;
	my $tags = shift;
	@$tags = map ( $self->colored_tag($_),@$tags );
	return 1;
}

sub insert_color {
	my $self = shift;
	my $tag = shift;
	my @color = split(/ /,shift);
	require Term::ANSIColor;
	foreach my $color (@color) {
		return unless defined $Term::ANSIColor::attributes{$color};
	}
	my $sth = $self->dbh->prepare(
		'INSERT OR REPLACE INTO colors (tag,color) VALUES(?,?)');
	$sth->execute($tag,"@color") or warn $self->dbh->errstr && return;
	return 1;
}

sub color {
	# Takes tag as input; returns color
	my $self = shift;
	return unless $self->color_enabled;
	my $tag = shift;
	my $sth = $self->dbh->prepare('SELECT color FROM colors WHERE tag=?');
	$sth->execute($tag) or warn $self->dbh->errstr && return;
	my $color;
	while ( local @_ = $sth->fetchrow_array ) {
		$color = shift;
	}
	return unless $color;
	return $color;
}

sub remove_tag {
	# Deletes specified tags from specified file.
	my ( $self, $file, @tags ) = @_;
}

1;
